@@ -18,7 +18,7 @@ fn bench_verify_eval<E: Engine, const NUM_COEFFS: usize>(c: &mut Criterion) {
     for i in 0..NUM_COEFFS {
         coeffs[i] = rng.gen::<u64>().into();
     }
-    let polynomial: Polynomial<E> = Polynomial::new_from_coeffs(coeffs, NUM_COEFFS - 1);
+    let polynomial = Polynomial::new_from_coeffs(coeffs, NUM_COEFFS - 1);
     let mut prover = KZGProver::new(&params);
     let verifier = KZGVerifier::new(&params);
     let commitment = prover.commit(polynomial.clone());
@@ -41,5 +41,5 @@ fn bench_verify_eval<E: Engine, const NUM_COEFFS: usize>(c: &mut Criterion) {
     );
 }
 
-criterion_group!(verify_eval, bench_verify_eval<Bls12, 10>, bench_verify_eval<Bls12, 50>, bench_verify_eval<Bls12, 100>, bench_verify_eval<Bls12, 200>, bench_verify_eval<Bls12, 500>);
+criterion_group!(verify_eval, bench_verify_eval<Bls12, 10>, bench_verify_eval<Bls12, 50>, bench_verify_eval<Bls12, 100>, bench_verify_eval<Bls12, 200>);
 criterion_main!(verify_eval);
